@@ -25,60 +25,95 @@ static void dummy(int x) { dummy2(x-1); }
 #else
 
 #include "lf_functions.h"
-#ifndef HAVE_NS_TYPE
-#define HAVE_NS_TYPE
+#if !defined(HAVE_NS_TYPE) && defined(NS_INADDRSZ)
+# define HAVE_NS_TYPE
 #endif
 #include <spf2/spf.h>
 #include <spf2/spf_dns_resolv.h>
 #include <spf2/spf_dns_cache.h>
 
+extern SPF_dns_server_t * SPF_dns_exim_new(int);
+
+
 static void *
 spf_open(uschar *filename, uschar **errmsg)
 {
-  SPF_server_t *spf_server = NULL;
-  spf_server = SPF_server_new(SPF_DNS_CACHE, 0);
-  if (spf_server == NULL) {
-    *errmsg = US"SPF_server_new() failed";
-    return NULL;
+SPF_dns_server_t * dc;
+SPF_server_t *spf_server = NULL;
+int debug = 0;
+
+DEBUG(D_lookup) debug = 1;
+
+if ((dc = SPF_dns_exim_new(debug)))
+  if ((dc = SPF_dns_cache_new(dc, NULL, debug, 8)))
+    spf_server = SPF_server_new_dns(dc, debug);
+
+if (!spf_server)
+  {
+  *errmsg = US"SPF_dns_exim_nnew() failed";
+  return NULL;
   }
-  return (void *) spf_server;
+return (void *) spf_server;
 }
 
+
 static void
 spf_close(void *handle)
 {
-  SPF_server_t *spf_server = handle;
-  if (spf_server) SPF_server_free(spf_server);
+SPF_server_t *spf_server = handle;
+if (spf_server) SPF_server_free(spf_server);
 }
 
 static int
 spf_find(void *handle, uschar *filename, const uschar *keystring, int key_len,
              uschar **result, uschar **errmsg, uint *do_cache)
 {
-  SPF_server_t *spf_server = handle;
-  SPF_request_t *spf_request = NULL;
-  SPF_response_t *spf_response = NULL;
+SPF_server_t *spf_server = handle;
+SPF_request_t *spf_request;
+SPF_response_t *spf_response = NULL;
 
-  spf_request = SPF_request_new(spf_server);
-  if (spf_request == NULL) {
-    *errmsg = US"SPF_request_new() failed";
-    return FAIL;
+if (!(spf_request = SPF_request_new(spf_server)))
+  {
+  *errmsg = US"SPF_request_new() failed";
+  return FAIL;
   }
 
-  if (SPF_request_set_ipv4_str(spf_request, CS filename)) {
+#if HAVE_IPV6
+switch (string_is_ip_address(filename, NULL))
+#else
+switch (4)
+#endif
+  {
+  case 4:
+    if (!SPF_request_set_ipv4_str(spf_request, CS filename))
+      break;
+    *errmsg = string_sprintf("invalid IPv4 address '%s'", filename);
+    return FAIL;
+#if HAVE_IPV6
+
+  case 6:
+    if (!SPF_request_set_ipv6_str(spf_request, CS filename))
+      break;
+    *errmsg = string_sprintf("invalid IPv6 address '%s'", filename);
+    return FAIL;
+
+  default:
     *errmsg = string_sprintf("invalid IP address '%s'", filename);
     return FAIL;
-  }
-  if (SPF_request_set_env_from(spf_request, CS keystring)) {
-    *errmsg = string_sprintf("invalid envelope from address '%s'", keystring);
-    return FAIL;
+#endif
   }
 
-  SPF_request_query_mailfrom(spf_request, &spf_response);
-  *result = string_copy(US SPF_strresult(SPF_response_result(spf_response)));
-  SPF_response_free(spf_response);
-  SPF_request_free(spf_request);
-  return OK;
+if (SPF_request_set_env_from(spf_request, CS keystring))
+    {
+  *errmsg = string_sprintf("invalid envelope from address '%s'", keystring);
+  return FAIL;
+}
+
+SPF_request_query_mailfrom(spf_request, &spf_response);
+*result = string_copy(US SPF_strresult(SPF_response_result(spf_response)));
+SPF_response_free(spf_response);
+SPF_request_free(spf_request);
+return OK;
 }
 
 
