@@ -107,16 +107,16 @@ static struct scan
 void
 features_malware(void)
 {
-const struct scan * sc;
 const uschar * s;
 uschar * t;
 uschar buf[64];
 
 spf(buf, sizeof(buf), US"_HAVE_MALWARE_");
 
-for (sc = m_scans; sc->scancode != -1; sc++)
+for (const struct scan * sc = m_scans; sc->scancode != -1; sc++)
   {
-  for(s = sc->name, t = buf+14; *s; s++) if (*s != '-') *t++ = toupper(*s);
+  for (s = sc->name, t = buf+14; *s; s++) if (*s != '-')
+    *t++ = toupper(*s);
   *t = '\0';
   builtin_macro_create(buf);
   }
@@ -349,13 +349,13 @@ return cre;
          -2 on timeout or error
 */
 static int
-recv_line(int fd, uschar * buffer, int bsize, int tmo)
+recv_line(int fd, uschar * buffer, int bsize, time_t tmo)
 {
 uschar * p = buffer;
 ssize_t rcv;
 BOOL ok = FALSE;
 
-if (!fd_ready(fd, tmo-time(NULL)))
+if (!fd_ready(fd, tmo))
   return -2;
 
 /*XXX tmo handling assumes we always get a whole line */
@@ -382,9 +382,9 @@ return p - buffer;
 
 /* return TRUE iff size as requested */
 static BOOL
-recv_len(int sock, void * buf, int size, int tmo)
+recv_len(int sock, void * buf, int size, time_t tmo)
 {
-return fd_ready(sock, tmo-time(NULL))
+return fd_ready(sock, tmo)
   ? recv(sock, buf, size, 0) == size
   : FALSE;
 }
@@ -430,7 +430,7 @@ for (;;)
 }
 
 static inline int
-mksd_read_lines (int sock, uschar *av_buffer, int av_buffer_size, int tmo)
+mksd_read_lines (int sock, uschar *av_buffer, int av_buffer_size, time_t tmo)
 {
 client_conn_ctx cctx = {.sock = sock};
 int offset = 0;
@@ -438,7 +438,7 @@ int i;
 
 do
   {
-  i = ip_recv(&cctx, av_buffer+offset, av_buffer_size-offset, tmo-time(NULL));
+  i = ip_recv(&cctx, av_buffer+offset, av_buffer_size-offset, tmo);
   if (i <= 0)
     {
     (void) malware_panic_defer(US"unable to read from mksd UNIX socket (/var/run/mksd/socket)");
@@ -497,7 +497,7 @@ switch (*line)
 
 static int
 mksd_scan_packed(struct scan * scanent, int sock, const uschar * scan_filename,
-  int tmo)
+  time_t tmo)
 {
 struct iovec iov[3];
 const char *cmd = "MSQ\n";
@@ -746,7 +746,7 @@ if (!malware_ok)
       if (m_sock_send(malware_daemon_ctx.sock, scanrequest, Ustrlen(scanrequest), &errstr) < 0)
         return m_panic_defer(scanent, CUS callout_address, errstr);
 
-      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo-time(NULL));
+      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo);
 
       if (bread <= 0)
         return m_panic_defer_3(scanent, CUS callout_address,
@@ -836,7 +836,7 @@ badseek:  err = errno;
 	    malware_daemon_ctx.sock);
 	  }
 
-	if (!(drweb_fbuf = US malloc(fsize_uint)))
+	if (!(drweb_fbuf = store_malloc(fsize_uint)))
 	  {
 	  (void)close(drweb_fd);
 	  return m_panic_defer_3(scanent, NULL,
@@ -849,7 +849,7 @@ badseek:  err = errno;
 	  {
 	  int err = errno;
 	  (void)close(drweb_fd);
-	  free(drweb_fbuf);
+	  store_free(drweb_fbuf);
 	  return m_panic_defer_3(scanent, NULL,
 	    string_sprintf("can't read spool file %s: %s",
 	      eml_filename, strerror(err)),
@@ -860,11 +860,12 @@ badseek:  err = errno;
 	/* send file body to socket */
 	if (send(malware_daemon_ctx.sock, drweb_fbuf, fsize, 0) < 0)
 	  {
-	  free(drweb_fbuf);
+	  store_free(drweb_fbuf);
 	  return m_panic_defer_3(scanent, CUS callout_address, string_sprintf(
 	    "unable to send file body to socket (%s)", scanner_options),
 	    malware_daemon_ctx.sock);
 	  }
+	store_free(drweb_fbuf);
 	}
       else
 	{
@@ -898,7 +899,6 @@ badseek:  err = errno;
       /* "virus(es) found" if virus number is > 0 */
       if (drweb_vnum)
 	{
-	int i;
 	gstring * g = NULL;
 
 	/* setup default virus name */
@@ -909,7 +909,7 @@ badseek:  err = errno;
 	  drweb_re = m_pcre_compile(drweb_re_str, &errstr);
 
 	/* read and concatenate virus names into one string */
-	for (i = 0; i < drweb_vnum; i++)
+	for (int i = 0; i < drweb_vnum; i++)
 	  {
 	  int ovector[10*3];
 
@@ -918,7 +918,9 @@ badseek:  err = errno;
 	    return m_panic_defer_3(scanent, CUS callout_address,
 			      US"cannot read report size", malware_daemon_ctx.sock);
 	  drweb_slen = ntohl(drweb_slen);
-	  tmpbuf = store_get(drweb_slen);
+
+	  /* assume tainted, since it is external input */
+	  tmpbuf = store_get(drweb_slen, TRUE);
 
 	  /* read report body */
 	  if (!recv_len(malware_daemon_ctx.sock, tmpbuf, drweb_slen, tmo))
@@ -1044,7 +1046,7 @@ badseek:  err = errno;
 #ifndef DISABLE_MAL_FSECURE
     case M_FSEC: /* "fsecure" scanner type ---------------------------------- */
       {
-      int i, j, bread = 0;
+      int i, bread = 0;
       uschar * file_name;
       uschar av_buffer[1024];
       static uschar *cmdopt[] = { US"CONFIGURE\tARCHIVE\t1\n",
@@ -1064,13 +1066,13 @@ badseek:  err = errno;
 	if (m_sock_send(malware_daemon_ctx.sock, cmdopt[i], Ustrlen(cmdopt[i]), &errstr) < 0)
 	  return m_panic_defer(scanent, CUS callout_address, errstr);
 
-	bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo-time(NULL));
+	bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo);
 	if (bread > 0) av_buffer[bread]='\0';
 	if (bread < 0)
 	  return m_panic_defer_3(scanent, CUS callout_address,
 	    string_sprintf("unable to read answer %d (%s)", i, strerror(errno)),
 	    malware_daemon_ctx.sock);
-	for (j = 0; j < bread; j++)
+	for (int j = 0; j < bread; j++)
 	  if (av_buffer[j] == '\r' || av_buffer[j] == '\n')
 	    av_buffer[j] ='@';
 	}
@@ -1096,7 +1098,7 @@ badseek:  err = errno;
 	  {
 	  errno = ETIMEDOUT;
 	  i =  av_buffer+sizeof(av_buffer)-p;
-	  if ((bread= ip_recv(&malware_daemon_ctx, p, i-1, tmo-time(NULL))) < 0)
+	  if ((bread= ip_recv(&malware_daemon_ctx, p, i-1, tmo)) < 0)
 	    return m_panic_defer_3(scanent, CUS callout_address,
 	      string_sprintf("unable to read result (%s)", strerror(errno)),
 	      malware_daemon_ctx.sock);
@@ -1401,7 +1403,7 @@ badseek:  err = errno;
 
       /* wait for result */
       memset(av_buffer, 0, sizeof(av_buffer));
-      if ((bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo-time(NULL))) <= 0)
+      if ((bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo)) <= 0)
 	return m_panic_defer_3(scanent, CUS callout_address,
 	  string_sprintf("unable to read from UNIX socket (%s)", scanner_options),
 	  malware_daemon_ctx.sock);
@@ -1464,7 +1466,7 @@ badseek:  err = errno;
 	/* Local file; so we def want to use_scan_command and don't want to try
 	 * passing IP/port combinations */
 	use_scan_command = TRUE;
-	cd = (clamd_address *) store_get(sizeof(clamd_address));
+	cd = (clamd_address *) store_get(sizeof(clamd_address), FALSE);
 
 	/* extract socket-path part */
 	sublist = scanner_options;
@@ -1498,7 +1500,7 @@ badseek:  err = errno;
 	    continue;
 	    }
 
-	  cd = (clamd_address *) store_get(sizeof(clamd_address));
+	  cd = (clamd_address *) store_get(sizeof(clamd_address), FALSE);
 
 	  /* extract host and port part */
 	  sublist = scanner_options;
@@ -1667,7 +1669,7 @@ b_seek:   err = errno;
 	if (lseek(clam_fd, 0, SEEK_SET) < 0)
 	  goto b_seek;
 
-	if (!(clamav_fbuf = US malloc(fsize_uint)))
+	if (!(clamav_fbuf = store_malloc(fsize_uint)))
 	  {
 	  (void)close(clam_fd);
 	  return m_panic_defer_3(scanent, NULL,
@@ -1679,7 +1681,7 @@ b_seek:   err = errno;
 	if ((result = read(clam_fd, clamav_fbuf, fsize_uint)) < 0)
 	  {
 	  int err = errno;
-	  free(clamav_fbuf); (void)close(clam_fd);
+	  store_free(clamav_fbuf); (void)close(clam_fd);
 	  return m_panic_defer_3(scanent, NULL,
 	    string_sprintf("can't read spool file %s: %s",
 	      eml_filename, strerror(err)),
@@ -1694,13 +1696,12 @@ b_seek:   err = errno;
 	    (send(malware_daemon_ctx.sock, clamav_fbuf, fsize_uint, 0) < 0) ||
 	    (send(malware_daemon_ctx.sock, &send_final_zeroblock, sizeof(send_final_zeroblock), 0) < 0))
 	  {
-	  free(clamav_fbuf);
+	  store_free(clamav_fbuf);
 	  return m_panic_defer_3(scanent, NULL,
 	    string_sprintf("unable to send file body to socket (%s)", hostname),
 	    malware_daemon_ctx.sock);
 	  }
-
-	free(clamav_fbuf);
+	store_free(clamav_fbuf);
 	}
       else
 	{ /* use scan command */
@@ -1737,7 +1738,7 @@ b_seek:   err = errno;
 
       /* Read the result */
       memset(av_buffer, 0, sizeof(av_buffer));
-      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo-time(NULL));
+      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo);
       (void)close(malware_daemon_ctx.sock);
       malware_daemon_ctx.sock = -1;
       malware_daemon_ctx.tls_ctx = NULL;
@@ -1895,7 +1896,7 @@ b_seek:   err = errno;
 	return m_panic_defer(scanent, CUS callout_address, errstr);
 
       /* Read the result */
-      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo-time(NULL));
+      bread = ip_recv(&malware_daemon_ctx, av_buffer, sizeof(av_buffer), tmo);
 
       if (bread <= 0)
 	return m_panic_defer_3(scanent, CUS callout_address,
@@ -2282,9 +2283,8 @@ if (!fprot6d_re_virus)
 void
 malware_show_supported(FILE * f)
 {
-struct scan * sc;
 fprintf(f, "Malware:");
-for (sc = m_scans; sc->scancode != (scanner_t)-1; sc++) fprintf(f, " %s", sc->name);
+for (struct scan * sc = m_scans; sc->scancode != (scanner_t)-1; sc++) fprintf(f, " %s", sc->name);
 fprintf(f, "\n");
 }
 
