@@ -74,9 +74,6 @@
 /* Some parameter values */
 #define PDKIM_QUERYMETHOD_DNS_TXT 0
 
-/*#define PDKIM_ALGO_RSA_SHA256     0 */
-/*#define PDKIM_ALGO_RSA_SHA1       1 */
-
 #define PDKIM_CANON_SIMPLE        0
 #define PDKIM_CANON_RELAXED       1
 
@@ -142,7 +139,7 @@ typedef struct pdkim_signature {
   /* (v=) The version, as an integer. Currently, always "1" */
   int version;
 
-  /* (a=) The signature algorithm. Either PDKIM_ALGO_RSA_SHA256 */
+  /* (a=) The signature algorithm. */
   int keytype;	/* pdkim_keytypes index */
   int hashtype;	/* pdkim_hashes index */
 
@@ -280,7 +277,7 @@ typedef struct pdkim_ctx {
   pdkim_bodyhash *bodyhash;
 
   /* Callback for dns/txt query method (verification only) */
-  uschar * (*dns_txt_callback)(uschar *);
+  uschar * (*dns_txt_callback)(const uschar *);
 
   /* Coder's little helpers */
   gstring   *cur_header;
@@ -313,7 +310,7 @@ extern "C" {
 
 void	   pdkim_init         (void);
 
-void	   pdkim_init_context (pdkim_ctx *, BOOL, uschar * (*)(uschar *));
+void	   pdkim_init_context (pdkim_ctx *, BOOL, uschar * (*)(const uschar *));
 
 DLLEXPORT
 pdkim_signature *pdkim_init_sign    (pdkim_ctx *,
@@ -321,7 +318,7 @@ pdkim_signature *pdkim_init_sign    (pdkim_ctx *,
 			       const uschar **);
 
 DLLEXPORT
-pdkim_ctx *pdkim_init_verify  (uschar * (*)(uschar *), BOOL);
+pdkim_ctx *pdkim_init_verify  (uschar * (*)(const uschar *), BOOL);
 
 DLLEXPORT
 void       pdkim_set_optional (pdkim_signature *, char *, char *,int, int,
