@@ -11,7 +11,7 @@
 #ifdef SUPPORT_SPF
 
 /* Yes, we do have ns_type. spf.h redefines it if we don't set this. Doh */
-#ifndef HAVE_NS_TYPE
+#if !defined(HAVE_NS_TYPE) && defined(NS_INADDRSZ)
 # define HAVE_NS_TYPE
 #endif
 #include <spf2/spf.h>
@@ -25,7 +25,8 @@ typedef struct spf_result_id {
 } spf_result_id;
 
 /* prototypes */
-BOOL spf_init(uschar *,uschar *);
+BOOL spf_init(void);
+BOOL spf_conn_init(uschar *, uschar *);
 int  spf_process(const uschar **, uschar *, int);
 
 #define SPF_PROCESS_NORMAL  0
